<?php
namespace SGalinski\Tinymce4Rte\Form\Element;

/*
 * This file is part of the TYPO3 CMS project.
 *
 * It is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License, either version 2
 * of the License, or any later version.
 *
 * For the full copyright and license information, please read the
 * LICENSE.txt file that was distributed with this source code.
 *
 * The TYPO3 project - inspiring people to share!
 */

use TYPO3\CMS\Backend\Form\Element\AbstractFormElement;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Authentication\BackendUserAuthentication;
use TYPO3\CMS\Core\Database\DatabaseConnection;
use TYPO3\CMS\Core\Html\RteHtmlParser;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\MathUtility;
use TYPO3\CMS\Core\Utility\PathUtility;
use SGalinski\Tinymce\Loader;

/**
 * Render rich text editor in FormEngine
 */
class RichTextElement extends AbstractFormElement {

	/**
	 * Main result array as defined in initializeResultArray() of AbstractNode
	 *
	 * @var array
	 */
	protected $resultArray;

	/**
	 * pid of page record the TSconfig is located at.
	 * This is pid of record if table is not pages, or uid if table is pages
	 *
	 * @var int
	 */
	protected $pidOfPageRecord;

	/**
	 * pid of fixed versioned record.
	 * This is the pid of the record in normal cases, but is changed to the pid
	 * of the "mother" record in case the handled record is a versioned overlay
	 * and "mother" is located at a different pid.
	 *
	 * @var int
	 */
	protected $pidOfVersionedMotherRecord;

	/**
	 * Native, not further processed TsConfig of RTE section for this record on given pid.
	 *
	 * Example:
	 *
	 * RTE = foo
	 * RTE.bar = xy
	 *
	 * array(
	 * 	'value' => 'foo',
	 * 	'properties' => array(
	 * 		'bar' => 'xy',
	 * 	),
	 * );
	 *
	 * @var array
	 */
	protected $vanillaRteTsConfig;

	/**
	 * Based on $vanillaRteTsConfig, this property contains "processed" configuration
	 * where table and type specific RTE setup is merged into 'default.' array.
	 *
	 * @var array
	 */
	protected $processedRteConfiguration;

	/**
	 * Parsed "defaultExtras" TCA
	 *
	 * @var array
	 */
	protected $defaultExtras;

	/**
	 * Indicates that the tinymce is loaded
	 *
	 * @var bool
	 */
	static protected $coreLoaded = FALSE;

	/**
	 * This will render a <textarea> OR RTE area form field,
	 * possibly with various control/validation features
	 *
	 * @return array As defined in initializeResultArray() of AbstractNode
	 */
	public function render() {
		$table = $this->globalOptions['table'];
		$fieldName = $this->globalOptions['fieldName'];
		$row = $this->globalOptions['databaseRow'];
		$parameterArray = $this->globalOptions['parameterArray'];

		$backendUser = $this->getBackendUserAuthentication();

		$this->resultArray = $this->initializeResultArray();
		$this->defaultExtras = BackendUtility::getSpecConfParts($parameterArray['fieldConf']['defaultExtras']);
		$this->pidOfPageRecord = $table === 'pages' && MathUtility::canBeInterpretedAsInteger($row['uid']) ? (int)$row['uid'] : (int)$row['pid'];
		BackendUtility::fixVersioningPid($table, $row);
		$this->pidOfVersionedMotherRecord = (int)$row['pid'];
		$this->vanillaRteTsConfig = $backendUser->getTSConfig('RTE', BackendUtility::getPagesTSconfig($this->pidOfPageRecord));
		$this->processedRteConfiguration = BackendUtility::RTEsetup(
			$this->vanillaRteTsConfig['properties'],
			$table,
			$fieldName,
			$this->globalOptions['recordTypeValue']
		);

		$this->resultArray['html'] = $this->getMainHtml($parameterArray, $row);

		return $this->resultArray;
	}

	/**
	 * Create main HTML elements
	 *
	 * @return string Main RTE html
	 */
	protected function getMainHtml($parameterArray, $row) {
		/** @var PageRenderer $pageRenderer */
		$pageRenderer = $GLOBALS['SOBE']->doc->getPageRenderer();

		// render the tinymce textarea
		$value = $this->transformDatabaseContentToEditor($parameterArray['itemFormElValue']);

		// This seems to result in:
		//	_TRANSFORM_bodytext (the handled field name) in case the field is a direct DB field
		//	_TRANSFORM_vDEF (constant string) in case the RTE is within a flex form
		$triggerFieldName = preg_replace('/\\[([^]]+)\\]$/', '[_TRANSFORM_\\1]', $parameterArray['itemFormElName']);

		// render RTE field
		$editorId = uniqid();
		$width = (GeneralUtility::_GP('M') === 'wizard_rte' ? '100%' : '650px');
		$code = '<input type="hidden" name="' . htmlspecialchars($triggerFieldName) . '" value="RTE" />';
		$code .= '<div style="width: ' . $width . '"><textarea id="editor' . $editorId . '" class="tinymce4_rte"
			name="' . htmlspecialchars($parameterArray['itemFormElName']) . '"
			rows="20" cols="100">' . GeneralUtility::formatForTextarea($value) . '</textarea></div>';

		// add the tinymce code and it's configuration
		if (!self::$coreLoaded) {
			self::$coreLoaded = TRUE;

			/** @var Loader $tinyMCE */
			$tinyMCE = GeneralUtility::makeInstance('SGalinski\\Tinymce\\Loader');
			$tinyMCE->loadConfiguration($this->processedRteConfiguration['tinymceConfiguration']);
			if ($this->processedRteConfiguration['contentCSS'] !== '') {
				$contentCssFile = GeneralUtility::getFileAbsFileName(
					$this->processedRteConfiguration['contentCSS']
				);

				if (is_file($contentCssFile)) {
					$contentCssFile = GeneralUtility::getIndpEnv('TYPO3_SITE_URL') .
						PathUtility::stripPathSitePrefix($contentCssFile) . '?' . filemtime($contentCssFile);

					$tinyMCE->addConfigurationOption('content_css', $contentCssFile);
				}
			}

			$tinyMCE->addConfigurationOption(
				'changeMethod', 'function() {
					var TBE_EDITOR = window.TBE_EDITOR || null;
					if (TBE_EDITOR && TBE_EDITOR.fieldChanged && typeof TBE_EDITOR.fieldChanged === \'function\') {
						TBE_EDITOR.fieldChanged();
					}
				}'
			);
			$tinyMCE->loadJsViaPageRenderer($pageRenderer, TRUE);

			$imageModule = BackendUtility::getModuleUrl('rtehtmlarea_wizard_select_image');
			$linkModule = BackendUtility::getModuleUrl('rtehtmlarea_wizard_browse_links');
			$pageRenderer->addJsInlineCode(
				'RTEbasic',
				'window.RTEarea = window.RTEarea || {};
				window.RTEarea.linkToImageModule = ' . GeneralUtility::quoteJSvalue($imageModule) . ';
				window.RTEarea.linkToLinkModule = ' . GeneralUtility::quoteJSvalue($linkModule) . ';'
			);
		}

		// add RTE specific configuration data
		$languageId = max($row['sys_language_uid'], 0);
		$language = ($GLOBALS['LANG']->lang === '' ? 'default' : $GLOBALS['LANG']->lang);
		$pageRenderer->addJsInlineCode(
			'RTE' . $editorId,
			'window.RTEarea = window.RTEarea || {};
			window.RTEarea["editor' . $editorId . '"] = {};
			window.RTEarea["editor' . $editorId . '"].rteConfiguration = "' . rawurlencode($this->RTEtsConfigParams()) . '";
			window.RTEarea["editor' . $editorId . '"].typo3ContentLanguage = "' . $language . '";
			window.RTEarea["editor' . $editorId . '"].editorNo = "editor' . $editorId . '";
			window.RTEarea["editor' . $editorId . '"].sys_language_content = parseInt(' . $languageId . ');
		'
		);

		return $code;
	}

	/**
	 * A list of parameters that is mostly given as GET/POST to other RTE controllers.
	 *
	 * @return string
	 */
	protected function RTEtsConfigParams() {
		$parameters = BackendUtility::getSpecConfParametersFromArray($this->defaultExtras['rte_transform']['parameters']);
		$result = array(
			$this->globalOptions['table'],
			$this->globalOptions['databaseRow']['uid'],
			$this->globalOptions['fieldName'],
			$this->pidOfVersionedMotherRecord,
			$this->globalOptions['recordTypeValue'],
			$this->pidOfPageRecord,
			$parameters['imgpath'],
		);
		return implode(':', $result);
	}

	/**
	 * Performs transformation of content from database to richtext editor
	 *
	 * @param string $value Value to transform.
	 * @return string Transformed content
	 */
	protected function transformDatabaseContentToEditor($value) {
		// change <strong> to <b>
		$value = preg_replace('/<(\\/?)strong/i', '<$1b', $value);
		// change <em> to <i>
		$value = preg_replace('/<(\\/?)em([^b>]*>)/i', '<$1i$2', $value);

		if ($this->defaultExtras['rte_transform']) {
			$parameters = BackendUtility::getSpecConfParametersFromArray($this->defaultExtras['rte_transform']['parameters']);
			// There must be a mode set for transformation
			if ($parameters['mode']) {
				/** @var RteHtmlParser $parseHTML */
				$parseHTML = GeneralUtility::makeInstance(RteHtmlParser::class);
				$parseHTML->init($this->globalOptions['table'] . ':' . $this->globalOptions['fieldName'], $this->pidOfVersionedMotherRecord);
				$parseHTML->setRelPath('');
				$value = $parseHTML->RTE_transform($value, $this->defaultExtras, 'rte', $this->processedRteConfiguration);
			}
		}
		return $value;
	}

	/**
	 * @return BackendUserAuthentication
	 */
	protected function getBackendUserAuthentication() {
		return $GLOBALS['BE_USER'];
	}

}
